@@ -36,11 +36,13 @@ import socket
 
 from django.conf import settings
 from django.contrib.auth.models import User
+from django.contrib.sites.models import Site
 from django.db import models as db
 from django.utils.translation import ugettext_lazy as _
 
 from lck.cache import memoize
 from lck.django.common.models import TimeTrackable, WithConcurrentGetOrCreate
+from lck.django.choices import Choices
 
 
 ACTIVITYLOG_PROFILE_MODEL = getattr(settings, 'ACTIVITYLOG_PROFILE_MODEL',
@@ -88,7 +90,7 @@ class MonitoredActivity(db.Model):
 
 class UserAgent(TimeTrackable, WithConcurrentGetOrCreate):
     # those names can be over 350 characters in length
-    name = db.TextField(verbose_name=_("name"), unique=True)
+    name = db.TextField(verbose_name=_("name"), unique=True, db_index=True)
     profiles = db.ManyToManyField(ACTIVITYLOG_PROFILE_MODEL,
         verbose_name=_("profiles"), through="ProfileUserAgent", help_text="")
 
@@ -101,14 +103,14 @@ class UserAgent(TimeTrackable, WithConcurrentGetOrCreate):
 
 
 class IP(TimeTrackable, WithConcurrentGetOrCreate):
-    # FIXME: db_index anywhere?
     address = db.IPAddressField(verbose_name=_("IP address"),
         help_text=_("Presented as string."), unique=True,
-        blank=True, null=True, default=None)
+        blank=True, null=True, default=None, db_index=True)
     number = db.BigIntegerField(verbose_name=_("IP address"),
-        help_text=_("Presented as int."), editable=False, unique=True)
+        help_text=_("Presented as int."), editable=False, unique=True,
+        null=True, blank=True, default=None)
     hostname = db.CharField(verbose_name=_("hostname"), max_length=255,
-        unique=True, null=True, blank=True, default=None)
+        null=True, blank=True, default=None)
     profiles = db.ManyToManyField(ACTIVITYLOG_PROFILE_MODEL,
         verbose_name=_("profiles"), help_text="", through="ProfileIP")
 
@@ -132,6 +134,57 @@ class IP(TimeTrackable, WithConcurrentGetOrCreate):
         super(IP, self).save(*args, **kwargs)
 
 
+class BacklinkStatus(Choices):
+    _ = Choices.Choice
+
+    unknown = _("unknown")
+    verification_failed1 = _("single verification failed")
+    verification_failed2 = _("two verifications failed")
+    verification_failed3 = _("three verifications failed")
+    failed = _("verification terminally failed")
+    verified = _("verified")
+    merged = _("merged")
+
+    @classmethod
+    @Choices.ToIDs
+    def is_verifiable(cls):
+        return (cls.unknown, cls.verification_failed1, cls.verification_failed2,
+            cls.verification_failed3)
+
+    @classmethod
+    @Choices.ToIDs
+    def is_partially_failed(cls):
+        return (cls.verification_failed1, cls.verification_failed2,
+            cls.verification_failed3)
+
+    @classmethod
+    @Choices.ToIDs
+    def can_increment_failure_status(cls):
+        return (cls.unknown, cls.verification_failed1, cls.verification_failed2)
+
+    @classmethod
+    @Choices.ToIDs
+    def is_verified(cls):
+        return (cls.verified, cls.merged)
+
+
+class Backlink(TimeTrackable, WithConcurrentGetOrCreate):
+    site = db.ForeignKey(Site, verbose_name=_("site"))
+    url = db.URLField(verbose_name=_("URL"), max_length=500)
+    referrer = db.URLField(verbose_name=_("referrer"), max_length=500)
+    visits = db.PositiveIntegerField(verbose_name=_("visits"), default=1)
+    status = db.PositiveIntegerField(verbose_name=_("status"),
+        choices=BacklinkStatus(), default=BacklinkStatus.unknown.id)
+
+    class Meta:
+        verbose_name = _("backlink")
+        verbose_name_plural = _("backlinks")
+        unique_together = ('site', 'url', 'referrer')
+
+    def __unicode__(self):
+        return self.url
+
+
 class M2M(TimeTrackable, WithConcurrentGetOrCreate):
     user = db.ForeignKey(User, verbose_name=_("user")) # for Django-admin
     profile = db.ForeignKey(ACTIVITYLOG_PROFILE_MODEL,
