@@ -35,6 +35,7 @@ from __future__ import unicode_literals
 
 from collections import defaultdict
 from datetime import datetime
+from functools import partial
 from hashlib import sha256
 import re
 
@@ -47,6 +48,7 @@ from django.db import IntegrityError, transaction
 from django.db import models as db
 from django.forms import fields
 from django.template.defaultfilters import urlencode
+from django.utils.translation import ugettext
 from django.utils.translation import ugettext_lazy as _
 
 from lck.django.choices import Language
@@ -61,7 +63,7 @@ DEFAULT_SAVE_PRIORITY = getattr(settings, 'DEFAULT_SAVE_PRIORITY', 0)
 class Named(db.Model):
     """Describes an abstract model with a unique ``name`` field."""
 
-    name = db.CharField(verbose_name=_("name"), max_length=50, unique=True,
+    name = db.CharField(verbose_name=_("name"), max_length=75, unique=True,
         db_index=True)
 
     class Meta:
@@ -78,7 +80,7 @@ class Named(db.Model):
     class NonUnique(db.Model):
         """Describes an abstract model with a non-unique ``name`` field."""
 
-        name = db.CharField(verbose_name=_("name"), max_length=50)
+        name = db.CharField(verbose_name=_("name"), max_length=75)
 
         class Meta:
             abstract = True
@@ -335,6 +337,29 @@ class SavePrioritized(TimeTrackable):
         # FIXME: should this restore the values that were changed or not?
 
 
+class ImageModel(Titled, Slugged, TimeTrackable):
+    """Describes image objects. Usage::
+
+        class Icon(ImageModel):
+            image = ImageModel.image_field(upload_to='icons', etc.)
+    """
+    height = db.PositiveIntegerField(verbose_name=_("height"), default=0,
+            editable=False)
+    width = db.PositiveIntegerField(verbose_name=_("width"), default=0,
+            editable=False)
+    image_field = partial(db.ImageField, verbose_name=_("file"),
+            height_field='height', width_field='width', max_length=200)
+
+    def __unicode__(self):
+        if not self.image:
+            return ugettext("new image")
+        format = self.title, self.width, self.height, self.image.size/1024
+        return "%s (%dx%d, %d kB)" % format
+
+    class Meta:
+        abstract = True
+
+
 class EditorTrackable(db.Model):
     """Describes objects authored by users of the application. In the admin,
     on object creation the ``created_by`` field is set according to the editor.
